@@ -25,7 +25,7 @@
   * @author Samuel Mimram
   *)
 
-(* $Id: ssl.mli 2687 2006-09-10 10:40:18Z smimram $ *)
+(* $Id: ssl.mli 7933 2010-11-09 17:38:05Z smimram $ *)
 
 (** {1 Exceptions and errors} *)
 
@@ -114,8 +114,11 @@ exception Verify_error of verify_error
 
 (** {1 Communication} *)
 
-(** Initialize SSL functions. Should be called before calling any other function. *)
-val init : unit -> unit
+(** Initialize SSL functions. Should be called before calling any other
+    function. The parameter [thread_safe] should be set to true if you use
+    threads in you application (the same effect can achived by calling
+    [Ssl_threads.init] first. *)
+val init : ?thread_safe:bool -> unit -> unit
 
 (** Retrieve a human-readable message that corresponds to the last error that occurred. *)
 val get_error_string : unit -> string
@@ -134,15 +137,10 @@ type socket
 (** {2 Threads} *)
 
 (** You should not have to use those functions. They are only here for internal
-  * use (they are needed to make the openssl library thread-safe, see the
-  * [Ssl_threads] module). *)
-
-val crypto_num_locks : unit -> int
-
-val thread_id_function : (unit -> int) option ref
-
-val thread_locking_function : (int -> bool -> unit) option ref
+    * use (they are needed to make the openssl library thread-safe, see the *
+    [Ssl_threads] module). *)
 
+val thread_safe : bool ref
 
 (** {2 Contexts} *)
 
